from pathlib import Path
import shutil
import datetime
import jdatetime
import mimetypes


class File:

    ALLOWED_FORMATTERS = ['name', 'ext', 'type']

    def __init__(self, path: Path):
        if not path.is_file():
            raise ValueError
        self.path = path

    @property
    def name(self):
        return self.path.name

    @property
    def suffix(self):
        return self.path.suffix

    @property
    def ext(self):
        suffix = self.suffix
        if len(suffix) > 0:
            return suffix[1:]
        else:
            return 'unknown'

    @property
    def type(self):
        t = mimetypes.guess_type(self.name)[0]
        if t:
            t = t.split('/')[0]
            return t
        return 'unknown'

    @property
    def modified_date(self):
        mtime = datetime.datetime.fromtimestamp(self.path.stat().st_mtime)
        return mtime

    def copy(self, path: Path):
        shutil.copy2(self.path, path)

    def move(self, path: Path):
        shutil.move(self.path, path)

    @property
    def formatters_dict(self):
        dic = {}
        for formatter in self.ALLOWED_FORMATTERS:
            dic[formatter] = getattr(self, formatter)
        return dic

    def format(self, string: str, jalali_date: bool):
        if jalali_date:
            date = jdatetime.datetime.fromgregorian(datetime=self.modified_date)
        else:
            date = self.modified_date
        string = date.strftime(string)
        return string.format(**self.formatters_dict)

    def __str__(self):
        return self.path.name



from typing import Tuple
import locale
from utils import *
import click


@click.command()
@click.option("-i", "--input", "input_path",
              help="Input directory or file",
              required=True,
              type=click.Path(
                  exists=True,
                  path_type=Path
              ))
@click.option("-o", "--output", "output_path",
              help="Output directory",
              default="",
              type=click.Path(
                  file_okay=False,
                  writable=True,
                  path_type=Path
              ))
@click.option("-f", "--format", "format_pattern",
              help="Folders format",
              default="%Y/%B/%d/{type}",
              show_default=True)
@click.option("-e", "--exclude", "exclude_patterns",
              help="Excluding patterns",
              default=[],
              multiple=True)
@click.option("-l", "--locale", "locale_code",
              help="Date and time locale",
              default="en_US",
              show_default=True)
@click.option("-m", "--move", "should_move",
              help="Move input file(s) instead of copy",
              default=False,
              show_default=True,
              is_flag=True)
@click.option("-a", "--all", "all_files",
              help="Apply on all files including HIDDEN files",
              default=False,
              show_default=True,
              is_flag=True)
@click.option("-j", "--jalali", "jalali_date",
              help="Use Jalali calendar",
              default=False,
              show_default=True,
              is_flag=True)
def main(input_path: Path,
         output_path: Path,
         format_pattern: str,
         exclude_patterns: Tuple[str],
         locale_code: str,
         should_move: bool,
         all_files: bool,
         jalali_date: bool):

    exclude_patterns = list(exclude_patterns)
    if not all_files:
        exclude_patterns.append('**/.*')

    files = []
    if input_path.is_file():
        files = [File(input_path)]
    if input_path.is_dir():
        files = find_sub_files(input_path, exclude_patterns)

    locale.setlocale(locale.LC_ALL, locale_code)
    for file in files:
        path = output_path / file.format(format_pattern, jalali_date)
        path.mkdir(parents=True, exist_ok=True)
        if should_move:
            click.echo(f"[-] Moving {file.path} to {path}")
            file.move(path)
        else:
            click.echo(f"[-] Copying {file.path} to {path}")
            file.copy(path)


# Press the green button in the gutter to run the script.
if __name__ == '__main__':
    main()





from setuptools import setup

setup(
    name='Tidier',
    version='0.3',
    packages=['tidier'],
    url='https://github.com/itsamirhn/Tidier',
    download_url='https://github.com/itsamirhn/Tidier/archive/refs/tags/v0.2.tar.gz',
    license='MIT',
    author='AmirMohammad Hosseini Nasab',
    author_email='awmirhn@gmail.com',
    description='A tool for organizing files',
    install_requires=[
        'Click',
        'jdatetime',
    ],
    entry_points={
        'console_scripts': [
            'tidier=tidier.cli:main',
        ],
    },
)



from typing import List, Iterable

from core import *


def find_sub_files(path: Path, exclude_patterns: Iterable[str]) -> List[File]:
    all_sub_paths = path.glob('**/*')
    excluded_paths = []
    for pattern in exclude_patterns:
        excluded_paths += path.glob(pattern)
    files = [File(path) for path in all_sub_paths if path.is_file() and path not in excluded_paths]
    return files
